/* chipcal.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Calibrate a 7-filter group of MLSO/CHIP images.
 *------------------------------------------------------------------------------
 * Uses the "cfitsio" library to extract information from the FITS headers.
 * NOTE: The direction, scan time are obtained from the extension HDU.
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   21 June 2001
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include <ctype.h>
#include "math.h"
#include "/riker/d/sw/fits/cfitsio/fitsio.h"

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;
{

    float get_frac_offset () ;
    void prsum () ;
    void printerror () ;

    char *w ;				/* pointer to command line arguments */
    static char comment [80] ;
    static char file [80] ;
    static char radius_file [80] ;
    static char code_obs  [8] ;
    static char code_obs1 [8] ;
    static char code_obs2 [8] ;
    static char code_obs3 [8] ;
    static char code_obs4 [8] ;
    static char code_obs5 [8] ;
    static char code_obs6 [8] ;
    static char code_obs7 [8] ;
    static char lcode_obs [8] ;
    static char date_obs [24] ;		/* Observation date. */
    static char nampgm [24] ;
    static char object [24] ;
    static char origin [24] ;
    static char sdir2 [4] = "ru" ;	/* Scan direction: rd or ru. */
    static char telescop [24] ;
    static char instrume [24] ;
    static char type_obs [24] ;
    static char time_obs [24] ;		/* Observation time. */

    static long naxes [2] ;

    int anynull ;
    int bitpix = -32 ;
    int ihour, iminute, isecond ;	/* Begin scan time. */
    int iyear, imonth,  iday    ;	/* Begin date. */
    int ival ;
    int i, ix, iy ;
    int mapoff = 512 ;           /* color index offset (logical map #2). */
    int naxis = 2 ;
    int ncode = 0 ;
    int status = 0 ;
    int xdim, ydim ;

    long fpixel = 1 ;
    long npix ;

    float crradius ;			/* Solar Radius (pixels). */
    float datamin, datamax ;
    float *imgbuf ;			/* Pointer to image array. */
    float *img1, *img2, *img3, *img4, *img5, *img6, *img7 ;
    float nullval ;
    float radius ;
    float solar_p0 ;			/* P-angle. */
    float xcen, ycen ;
    float xdist, ydist ;
    float wavelen = 0.0 ;

    float *fbuf ;

    fitsfile *fptr ;

    if (argc <= 7) { prsum () ; exit (-1) ; }

    strcpy (nampgm, argv [0]) ;

    while (--argc)
	{
	strcpy (file, argv [argc]) ;

	/*** Open FITS file. ***/

	if ( fits_open_file (&fptr, file, READONLY, &status) )
	    printerror ( status );

	/*** Primary FITS header. ***/

	if (fits_read_key (fptr, TINT, "NAXIS1", &xdim,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TINT, "NAXIS2", &ydim,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "DATE-OBS", date_obs,
	    comment, &status))  printerror (status) ;
	sscanf (date_obs, "%04d-%02d-%02d", &iyear, &imonth, &iday) ;
	printf ("iyear: %d  imonth: %d  iday: %d\n", iyear, imonth, iday) ;

	if (fits_read_key (fptr, TSTRING, "TIME-OBS", time_obs,
	    comment, &status))  printerror (status) ;
	sscanf (time_obs, "%2d:%2d:%2d", &ihour, &iminute, &isecond) ;
	printf ("ihour: %d  iminute: %d  isecond: %d\n",
	    ihour, iminute, isecond) ;

	if (fits_read_key (fptr, TSTRING, "ORIGIN", origin,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "TELESCOP", telescop,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "INSTRUME", instrume,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "OBJECT", object,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "TYPE-OBS", type_obs,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "CODE-OBS", code_obs,
	    comment, &status))  printerror (status) ;

	ncode = strlen (code_obs) ;
	if (ncode > 7)  ncode = 7 ;
	for (i = 0; i < ncode; i++)
	    {
	    ival = code_obs [i] ;
	    lcode_obs [i] = (char) tolower (ival) ;
	    }

	if (fits_read_key (fptr, TFLOAT, "CRRADIUS", &crradius,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TFLOAT, "SOLAR_P0", &solar_p0,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TFLOAT, "WAVELEN", &wavelen,
	    comment, &status))  printerror (status) ;

	fprintf (stdout, "file: %s  code_obs: %s  wavelen: %8.2f\n",
		 file, code_obs, wavelen) ;

	/* Read FITS image data. */

	/* Note that even though the FITS images contains unsigned integer */
	/* pixel values (or more accurately, signed integer pixels with    */
	/* a bias of 32768),  this routine is reading the values into a    */
	/* float array.   Cfitsio automatically performs the datatype      */
	/* conversion in cases like this.                                  */

	npix     = xdim * ydim ;
	imgbuf   = (float *) malloc (npix * sizeof (float)) ;
	nullval  = 0 ; /* Don't check for null values in the image. */

	if ( fits_read_img (fptr, TFLOAT, fpixel, npix, &nullval, imgbuf,
			    &anynull, &status) )
	    printerror ( status ) ;

        printf ("code_obs [0]: %c\n", code_obs [0]) ;

	switch (code_obs [0])
            {
	    case 'D':
	    case 'd': img1 = imgbuf ;
		      printf ("case d\n") ;
		      strcpy (code_obs1, code_obs) ;
		      break ;
	    case 'C':
	    case 'c': img2 = imgbuf ;
		      printf ("case c\n") ;
		      strcpy (code_obs2, code_obs) ;
		      break ;
	    case 'B':
	    case 'b': img3 = imgbuf ;
		      printf ("case b\n") ;
		      strcpy (code_obs3, code_obs) ;
		      break ;
	    case 'H':
	    case 'h': img4 = imgbuf ;
		      printf ("case h\n") ;
		      strcpy (code_obs4, code_obs) ;
		      break ;
	    case 'R':
	    case 'r': img5 = imgbuf ;
		      printf ("case r\n") ;
		      strcpy (code_obs5, code_obs) ;
		      break ;
	    case 'S':
	    case 's': img6 = imgbuf ;
		      printf ("case s\n") ;
		      strcpy (code_obs6, code_obs) ;
		      break ;
	    case 'T':
	    case 't': img7 = imgbuf ;
		      printf ("case t\n") ;
		      strcpy (code_obs7, code_obs) ;
		      break ;
	    default:  continue ;
	    }


	/*** Close FITS file. ***/

	if ( fits_close_file (fptr, &status) )  printerror (status) ;

	/*** Set the magnitude of each pixel to the radial distance (Rsun) ***/

	status = 0 ;

        }
   exit (0) ;
}

/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- */

/* prsum ()
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Print usage summary.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

void
prsum ()
{
    static char help [] = "\
chipcal image1 image2 image3 image4 image5 image6 image7 \n\
\n\
Example:  chipcal  20010101.123456.chp.dsr.fts 20010101.123456.chp.csr.fts \n\
20010101.123456.chp.bsr.fts 20010101.123456.chp.hsr.fts \n\
20010101.123456.chp.rsr.fts 20010101.123456.chp.ssr.fts \n\
20010101.123456.chp.tsr.fts \n " ;

    fputs (help, stderr) ;
}

/* -------------------------------------------------------------------------- */

void 
printerror ( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
        {
        fits_report_error (stderr, status); /* print error report */

        exit ( status );    /* terminate the program, returning error status */
        }

    return;
}

/* -------------------------------------------------------------------------- */

