/* chipcp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copy a chip image.
 *------------------------------------------------------------------------------
 * Uses the "cfitsio" library to extract information from the FITS headers.
 * NOTE: The direction, scan time are obtained from the extension HDU.
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   26 June 2001
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/riker/d/sw/fits/cfitsio/fitsio.h"

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;
{

    extern unsigned char *crop_img () ;		/* Crop image. */
    extern unsigned char *rot () ;		/* Rotate image. */
    float get_frac_offset () ;
    void prsum () ;
    void printerror () ;

    char *w ;				/* pointer to command line arguments */
    static char comment [80] ;
    static char cnaxis1 [80] ;
    static char cnaxis2 [80] ;
    static char file [80] ;
    static char filerot [80] ;
    static char radius_file [80] ;
    static char code_obs  [8] ;
    static char code_obs1 [8] ;
    static char code_obs2 [8] ;
    static char code_obs3 [8] ;
    static char code_obs4 [8] ;
    static char code_obs5 [8] ;
    static char code_obs6 [8] ;
    static char code_obs7 [8] ;
    static char lcode_obs [8] ;
    static char date_obs [24] ;		/* Observation date. */
    static char nampgm [24] ;
    static char object [24] ;
    static char origin [24] ;
    static char sdir2 [4] = "ru" ;	/* Scan direction: rd or ru. */
    static char telescop [24] ;
    static char instrume [24] ;
    static char type_obs [24] ;
    static char time_obs [24] ;		/* Observation time. */

    static int  extbuf [168] ;		/* FITS extension data. */
    static long naxes [2] ;

    int anynull ;
    int bitpix = -32 ;
    int hdutype = 0 ;
    int ihour, iminute, isecond ;	/* Begin scan time. */
    int iyear, imonth,  iday    ;	/* Begin date. */
    int ival ;
    int i, ix, iy ;
    int morekeys = 0 ;		/* Additional keywords to be reserved. */
    int naxis = 2 ;
    int nbypix = 2 ;
    int nbyflt = 4 ;
    int ncode = 0 ;
    int nelimg ;
    int nelext = 168 ;
    int status = 0 ;
    int xdim, ydim ;
    int xdimsq = 936, ydimsq = 936 ;

    long fpixel = 1 ;
    long npix ;

    float crradius ;			/* Solar Radius (pixels). */
    float crpix1, crpix2 ;		/* Sun center (pixels). */
    float datamin, datamax ;
    short *imgbuf ;			/* Pointer to image array. */
    short *imgsq ;			/* Pointer to square image. */
    short *imgrot ;			/* Pointer to rotated image. */
    short *img1, *img2, *img3, *img4, *img5, *img6, *img7 ;
    float nullval = 0 ;			/* Don't check for null values in img.*/
    float radius ;
    float solar_p0 ;			/* P-angle. */
    float xcen, ycen ;
    float xcensq, ycensq ;
    float xshift, yshift ;		/* x/y shift for cropping. */
    float wavelen = 0.0 ;

    float *fbuf ;

    fitsfile *fp, *fprot ;

    if (argc <= 1) { prsum () ; exit (-1) ; }

    strcpy (nampgm, argv [0]) ;

    while (--argc)
	{
	strcpy (file, argv [argc]) ;

	/*** Open FITS file. ***/

	if ( fits_open_file (&fp, file, READONLY, &status) )
	    printerror ( status );

	/*** Primary FITS header. ***/

	if (fits_read_key (fp, TINT, "NAXIS1", &xdim,
	    cnaxis1, &status))  printerror (status) ;

	if (fits_read_key (fp, TINT, "NAXIS2", &ydim,
	    cnaxis2, &status))  printerror (status) ;

	if (fits_read_key (fp, TINT, "BITPIX", &bitpix,
	    comment, &status))  printerror (status) ;
	nbypix = bitpix / 8 ;

	if (fits_read_key (fp, TSTRING, "DATE-OBS", date_obs,
	    comment, &status))  printerror (status) ;
	sscanf (date_obs, "%04d-%02d-%02d", &iyear, &imonth, &iday) ;
	printf ("iyear: %d  imonth: %d  iday: %d\n", iyear, imonth, iday) ;

	if (fits_read_key (fp, TSTRING, "TIME-OBS", time_obs,
	    comment, &status))  printerror (status) ;
	sscanf (time_obs, "%2d:%2d:%2d", &ihour, &iminute, &isecond) ;
	printf ("ihour: %d  iminute: %d  isecond: %d\n",
	    ihour, iminute, isecond) ;

	if (fits_read_key (fp, TSTRING, "ORIGIN", origin,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "TELESCOP", telescop,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "INSTRUME", instrume,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "OBJECT", object,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "TYPE-OBS", type_obs,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "CODE-OBS", code_obs,
	    comment, &status))  printerror (status) ;

	ncode = strlen (code_obs) ;
	if (ncode > 7)  ncode = 7 ;
	for (i = 0; i < ncode; i++)
	    {
	    ival = code_obs [i] ;
	    lcode_obs [i] = (char) tolower (ival) ;
	    }

	if (fits_read_key (fp, TFLOAT, "CRPIX1", &crpix1,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "CRPIX2", &crpix2,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "CRRADIUS", &crradius,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "SOLAR_P0", &solar_p0,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "WAVELEN", &wavelen,
	    comment, &status))  printerror (status) ;

	fprintf (stdout, "file: %s  code_obs: %s  wavelen: %8.2f\n",
		 file, code_obs, wavelen) ;

	/* Read FITS image data. */

	/* Note that even though the FITS images contains unsigned integer */
	/* pixel values (or more accurately, signed integer pixels with    */
	/* a bias of 32768),  this routine is reading the values into a    */
	/* float array.   Cfitsio automatically performs the datatype      */
	/* conversion in cases like this.                                  */

	if (crpix1 >= 0.0)
	    xcen = crpix1 ;
	else
	    xcen = (xdim - 1) * 0.5 ;

	if (crpix2 >= 0.0)
	    ycen = crpix2 ;
	else
	    ycen = (ydim - 1) * 0.5 ;

	xcensq = (xdim - 1) * 0.5 ;
	ycensq = (ydim - 1) * 0.5 ;

	npix     = xdim * ydim ;
	imgbuf   = (short *) malloc (npix * sizeof (short)) ;
	nullval  = 0 ; /* Don't check for null values in the image. */

	if ( fits_read_img (fp, TSHORT, fpixel, npix, &nullval, imgbuf,
			    &anynull, &status) )
	    printerror ( status ) ;

	/*** Crop image: [xdimsq, ydimsq]. ***/

/*
 *       imgsq = (short *) crop_img (imgbuf, xdim, ydim, nbypix, xshift, yshift,
 *			  xdimsq, ydimsq) ;
 */

	/*** Rotate image to correct for P-angle. ***/

/*
 *	imgrot = (short *) rot (imgsq, nbypix, xdimsq, ydimsq,
 *		  xcensq, ycensq, solar_p0) ;
 */

	/*** Create output file name. ***/

	sprintf (filerot, "%04d%02d%02d.%02d%02d%02d.chp.%s.cp.fts",
		 iyear, imonth, iday, ihour, iminute, isecond, lcode_obs) ;

	fprintf (stdout, "filerot: %s\n", filerot) ;

	/*** Create output file. ***/

	if ( fits_create_template (&fprot, filerot, file, &status) )
	    printerror ( status ) ;

	/*** Write square, rotated image array into fits data array. ***/

	nelimg = xdim * ydim ;
	if ( fits_write_img (fprot, TSHORT, fpixel, nelimg, imgbuf, &status) )
	    printerror ( status ) ;

/*
	nelimg = xdimsq * ydimsq ;
	if ( fits_write_img (fprot, TSHORT, fpixel, nelimg, imgrot, &status) )
	    printerror ( status ) ;
 */

	/*** Resize image (updates xdim, ydim, bitpix). ***/

/*
	naxes [0] = xdimsq ;
	naxes [1] = ydimsq ;
	if (fits_resize_img (fprot, 16, 2, naxes, &status) )
	    printerror ( status ) ;
 */

	if (fits_close_file (fprot, &status))  printerror ( status ) ;

	/*** Identify image according to wavelength. ***/

        printf ("code_obs [0]: %c\n", code_obs [0]) ;

	switch (code_obs [0])
            {
	    case 'D':
	    case 'd': img1 = imgrot ;
		      printf ("case d\n") ;
		      strcpy (code_obs1, code_obs) ;
		      break ;
	    case 'C':
	    case 'c': img2 = imgrot ;
		      printf ("case c\n") ;
		      strcpy (code_obs2, code_obs) ;
		      break ;
	    case 'B':
	    case 'b': img3 = imgrot ;
		      printf ("case b\n") ;
		      strcpy (code_obs3, code_obs) ;
		      break ;
	    case 'H':
	    case 'h': img4 = imgrot ;
		      printf ("case h\n") ;
		      strcpy (code_obs4, code_obs) ;
		      break ;
	    case 'R':
	    case 'r': img5 = imgrot ;
		      printf ("case r\n") ;
		      strcpy (code_obs5, code_obs) ;
		      break ;
	    case 'S':
	    case 's': img6 = imgrot ;
		      printf ("case s\n") ;
		      strcpy (code_obs6, code_obs) ;
		      break ;
	    case 'T':
	    case 't': img7 = imgrot ;
		      printf ("case t\n") ;
		      strcpy (code_obs7, code_obs) ;
		      break ;
	    default:  continue ;
	    }

	/*** Close FITS file. ***/

	if ( fits_close_file (fp, &status) )  printerror (status) ;
	status = 0 ;
        }

   
   exit (0) ;
}

/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- */

/* prsum ()
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Print usage summary.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

void
prsum ()
{
    static char help [] = "\
chipcal image1 image2 image3 image4 image5 image6 image7 \n\
\n\
Example:  chipcal  20010101.123456.chp.dsr.fts 20010101.123456.chp.csr.fts \n\
20010101.123456.chp.bsr.fts 20010101.123456.chp.hsr.fts \n\
20010101.123456.chp.rsr.fts 20010101.123456.chp.ssr.fts \n\
20010101.123456.chp.tsr.fts \n " ;

    fputs (help, stderr) ;
}

/* -------------------------------------------------------------------------- */

void 
printerror ( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
        {
        fits_report_error (stderr, status); /* print error report */

        exit ( status );    /* terminate the program, returning error status */
        }

    return;
}

/* -------------------------------------------------------------------------- */

