#include <stdio.h>
#include <math.h>
#include "LDmodel.h"

/*     This program does a least squares fit to solve the standard
       limb dark equation:

       (Brightness at R)/(BrightnessDiskCenter)
                                 = a + b*cos(R) + c*(1.-cos(R)*ln(1.+1./cos(R)))
    or
        B/BC = a + b*cos(R) + c*(1.-cos(R)*ln(1.+1./cos(R)))

       where R = arcsin( radialpositionpixels / Rsunpixels )  

       and   c = (1. -a -b ) / (0.306853 )

       Multiplying by BC we get:
      
	    B = BC*a + BC*b*cos(R) + BC*c*(1.-cos(R)*ln(1.+1./cos(R)))
    
        Calling  x = cos(R) and    z = (1.-cos(R)*ln(1.+1./cos(R)))  we get:

	    B = BC*a + BC*b*x  + BC*c*z

        Since c = (1. -a -b ) / (0.306853 ) we can expand and combine terms:

	    B = BC*a*(1.-z) + BC*b*(x-z) + BC*z

        If we now redefine a=BC*a and b=BC*b and c=BC  and d=(1.-z) and e=(x-z)
		       we get:

	    B = a*d + b*e + c*z
    
        which is a convenient equation to do a least squares minimization on!
*/

float
solve_LD_curve(brights,radii,Npts,radpix,LDp) 
    double *brights, *radii;
    int Npts;
    float radpix;
    struct LDparams *LDp;
{
    
    int ii;
    float  BriAtCen;
    double x,y,a,b,c;
    double z,d,e,tye,tyd,tyz,tde,ted,tdz,tzd,tez,tze,tdd,tee,tzz;
    double dede,yyy,eee,zzz;
    double *rad, *bri;
    double ovradpix;
    int doprint=0;

    tye=tyd=tyz=tde=ted=tdz=tzd=tez=tze=tdd=tee=tzz=0.;

    ovradpix= 1./(double)radpix;

    rad = radii;
    bri = brights;
    for(ii=0;ii<Npts;ii++){
	x = *rad * ovradpix ;
	x = asin ( x ) ;
	x = cos  ( x ) ;

	 
	z = ( 1. - x * log( 1. + 1./x ) ) / ( 0.306853 ) ;
	d = 1. -z ;
	e = x  -z ;

	y =  *bri;
	 
	tye += (y*e) ;
	tyd += (y*d) ;
	tyz += (y*z) ;
	tde += (d*e) ;     ted += (d*e) ;
	tdz += (d*z) ;     tzd += (d*z) ;
	tez += (e*z) ;     tze += (e*z) ;
	tdd += (d*d) ;
	tee += (e*e) ;
	tzz += (z*z) ;

	rad++; bri++;
     
    }
 
    dede = ( tee - tde*tde/tdd ) ;
    yyy = tyd*tdz/tdd - tyz ;
    eee = tez - ted*tdz / tdd ;
    zzz = tzz - tzd*tzd / tdd ;
 
    c = -1.0* ( yyy + (tye - tyd*tde/tdd)*eee/dede ) ;
    c = c / ((tdz*tde/tdd - tze)*eee/dede + zzz ) ;
 
    b = ( tye - (tyd-c*tzd)*tde/tdd - c*tze ) / dede ;
    a = ( tyd - b*ted - c*tzd ) / tdd ;
 
    a = a/c ;
    b = b/c ;
    BriAtCen = (float) c ;
    if(doprint==1) fprintf(stderr,"A B C %.2f %.2f %.2f\n",a,b,c);

    LDp->a = a;
    LDp->b = b;
    LDp->c = c;

    return(BriAtCen);
}
