/* radius_img1.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read FITS image.
 * Generate an output file with pixels whose magnitude represent the radial
 * distance from sun center in Rsun units.
 *
 * Uses the "cfitsio" library to extract information from the FITS headers.
 * NOTE: The direction & scan time are obtained from the extension HDU.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   1 Feburary 2001
 *  3 Jul 2001 : Use crpix1 & crpix2 for sun center (if they are not negative).
 *               Otherwise, sun center is set equal to the array center.
 * 26 Jul 2001 : Read input file from "radius_filename".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <ctype.h>
#include "math.h"
#include "/riker/d/sw/fits/cfitsio/fitsio.h"

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;
{

    float get_frac_offset () ;
    void prsum () ;
    void printerror () ;

    char *w ;				/* pointer to command line arguments */
    static char comment [80] ;
    static char fits_file [80] ;
    static char radius_file [80] ;
    static char code_obs [24] ;
    static char lcode_obs [24] ;
    static char date_obs [24] ;		/* Observation date. */
    static char object [24] ;
    static char origin [24] ;
    static char telescop [24] ;
    static char instrume [24] ;
    static char type_obs [24] ;
    static char time_obs [24] ;		/* Observation time. */

    static long naxes [2] ;

    int bitpix = -32 ;
    int ihour, iminute, isecond ;	/* Begin scan time. */
    int iyear, imonth,  iday    ;	/* Begin date. */
    int ival ;
    int i, ix, iy ;
    int naxis = 2 ;
    int status = 0 ;
    int xdim, ydim ;

    long fpixel = 1 ;
    long npix ;

    float crradius ;			/* Solar Radius (pixels). */
    float crpix1, crpix2 ;		/* Sun center position. */
    float datamin, datamax ;
    float radius ;
    float solar_p0 ;			/* P-angle. */
    float xcen, ycen ;
    float xdist, ydist ;

    float *fbuf ;

    fitsfile *fptr ;
    FILE *FD ;

    /*** Read input filename from "radius_filename". ***/

    FD = fopen ("radius_filename", "r") ;
    fscanf (FD, "%s", fits_file) ;
    fclose (FD) ;

	/*** Open FITS file. ***/

	if ( fits_open_file (&fptr, fits_file, READONLY, &status) )
	    printerror ( status );

	/*** Primary FITS header: fetch keyword parameters. ***/

	if (fits_read_key (fptr, TINT, "NAXIS1", &xdim,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TINT, "NAXIS2", &ydim,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "DATE-OBS", date_obs,
	    comment, &status))  printerror (status) ;
	sscanf (date_obs, "%04d-%02d-%02d", &iyear, &imonth, &iday) ;
	printf ("iyear: %d  imonth: %d  iday: %d\n", iyear, imonth, iday) ;

	if (fits_read_key (fptr, TSTRING, "TIME-OBS", time_obs,
	    comment, &status))  printerror (status) ;
	sscanf (time_obs, "%2d:%2d:%2d", &ihour, &iminute, &isecond) ;
	printf ("ihour: %d  iminute: %d  isecond: %d\n",
	    ihour, iminute, isecond) ;

	if (fits_read_key (fptr, TSTRING, "ORIGIN", origin,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "TELESCOP", telescop,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "INSTRUME", instrume,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "OBJECT", object,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "TYPE-OBS", type_obs,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "CODE-OBS", code_obs,
	    comment, &status))  printerror (status) ;
	for (i = 0; i < strlen (code_obs); i++)
	    {
	    ival = code_obs [i] ;
	    lcode_obs [i] = (char) tolower (ival) ;
	    }

	if (fits_read_key (fptr, TFLOAT, "CRPIX1", &crpix1,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TFLOAT, "CRPIX2", &crpix2,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TFLOAT, "CRRADIUS", &crradius,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TFLOAT, "SOLAR_P0", &solar_p0,
	    comment, &status))  printerror (status) ;

	/*** Close FITS file. ***/

	if ( fits_close_file (fptr, &status) )  printerror (status) ;

	/*** Set the magnitude of each pixel to the radial distance (Rsun) ***/

	if (crpix1 < 0.0 || crpix2 < 0.0)
	    {
	    xcen = (xdim - 1) / 2.0 ;
	    ycen = (ydim - 1) / 2.0 ;
	    }
	else
	    {
	    xcen = crpix1 ;
	    ycen = crpix2 ;
	    }

	datamin =  1.0e+30 ;
	datamax = -1.0e+30 ;
	fbuf = (float *) malloc (xdim * ydim * sizeof (float)) ;

	for (iy = 0; iy < ydim; iy++)
	    for (ix = 0; ix < xdim; ix++)
		{
		xdist = (float) ix - xcen ;
		ydist = (float) iy - ycen ;
		radius = sqrt ((xdist * xdist) + (ydist * ydist)) ;
		radius /= crradius ;
		*(fbuf + ix + iy * xdim) = radius ;
		if (radius < datamin)  datamin = radius ;
		if (radius > datamax)  datamax = radius ;
		}

	status = 0 ;

	/*** Create output FITS file. ***/

	naxes [0] = xdim ;
	naxes [1] = ydim ;
	npix = xdim * ydim ;
	strcpy (type_obs, "RADIAL DISTANCE") ;
	sprintf (radius_file, "!%04d%02d%02d.%02d%02d%02d.chp.%s.rad.fts",
		 iyear, imonth, iday, ihour, iminute, isecond, lcode_obs) ;

	if (fits_create_file (&fptr, radius_file, &status))
	    printerror (status) ; /* Create new file. */

	if (fits_create_img (fptr, bitpix, naxis, naxes, &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TSTRING, "DATE-OBS", date_obs,
			     "OBSERVATION DATE: CCYY-MM-DD", &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TSTRING, "TIME-OBS", time_obs,
			     "OBSERVATION TIME: HH:MM:SS [UTC]", &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TSTRING, "ORIGIN", origin,
			 " MAUNA LOA SOLAR OBSERVATORY", &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TSTRING, "TELESCOP", telescop,
			 " CHROMOSPHERIC HELIUM-I IMAGING PHOTOMETER", &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TSTRING, "INSTRUME", instrume,
			 " KODAK MEGAPLUS 1.6 CCD", &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TSTRING, "OBJECT", object,
			 " RADIAL DISTANCE FROM SUN CENTER [RSUN]", &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TSTRING, "TYPE-OBS", type_obs,
			 " TYPE OF OBSERVATION [CHARACTER CODE]", &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TFLOAT, "CRRADIUS", &crradius,
			 "SOLAR RADIUS [PIXELS]", &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TFLOAT, "DATAMIN", &datamin,
			 "MINIMUM DATA VALUE", &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TFLOAT, "DATAMAX", &datamax,
			 "MAXIMUM DATA VALUE", &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TFLOAT, "DISPMIN", &datamin,
			 "MIN VALUE FOR DISPLAY [UNSCALED]", &status))
	    printerror (status) ;

	if (fits_update_key (fptr, TFLOAT, "DISPMAX", &datamax,
			 "MAX VALUE FOR DISPLAY [UNSCALED]", &status))
	    printerror (status) ;

	if (fits_write_img (fptr, TFLOAT, fpixel, npix, fbuf, &status))
	    printerror (status) ;

	if (fits_close_file (fptr, &status)) printerror (status) ;
   exit (0) ;
}


/* -------------------------------------------------------------------------- */

/* prsum ()
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Print usage summary.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

void
prsum ()
{
    static char help [] = "\
radius_img   fits_file1   fits_file2   fits_file3 . . . \n\
\n\
Example:   radius_img   20010101.123456.mk4.rpb.fts \n\
Example:   radius_img   20010101.123456.dpm.dsk.fts \n\
Example:   radius_img   20010101.123456.chp.hsr.fts \n\
\n\
" ;

       fputs (help, stderr) ;
}

/* -------------------------------------------------------------------------- */

void 
printerror ( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
        {
        fits_report_error (stderr, status); /* print error report */

        exit ( status );    /* terminate the program, returning error status */
        }

    return;
}

/* -------------------------------------------------------------------------- */

